@@ -8,8 +8,7 @@ Methods:
 
 Classes:
    Calculation: Base calculation class
-   EnergyCalculation: Typical GB/PB binding FE calculations. Handles all sander
-                      and mmpbsa_py_energy program calls
+   EnergyCalculation: Typical GB/PB binding FE calculations. Handles all sander program calls
    RISMCalculation: RISM binding FE calculation
    NmodeCalc: normal mode entropy calculation
    QuasiHarmCalc: Quasi-harmonic entropy calculation
@@ -32,15 +31,17 @@ Classes:
 #  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License    #
 #  for more details.                                                           #
 # ##############################################################################
+import logging
 
 from GMXMMPBSA.exceptions import CalcError
-from GMXMMPBSA.exceptions import GMXMMPBSA_ERROR, GMXMMPBSA_WARNING
+from GMXMMPBSA.exceptions import GMXMMPBSA_ERROR
 import os
 import sys
 import numpy as np
 import math
 
-#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
+
+# +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 
 class CalculationList(list):
     """ This contains the list of all calculations that need to be run """
@@ -78,8 +79,8 @@ class CalculationList(list):
                 # Start timer, run calculation, then stop the timer
                 if self.timer_keys[i] is not None:
                     self.timer.start_timer(self.timer_keys[i])
-                if self.labels[i]:
-                    f.write(self.labels[i] + '\n')
+                if self.labels[i] and rank == 0:
+                    logging.info(self.labels[i])
                 calc.setup()
                 calc.run(rank, stdout=stdout, stderr=stderr)
                 if self.timer_keys[i] is not None:
@@ -87,24 +88,26 @@ class CalculationList(list):
         finally:
             if own_handle: f.close()
 
-#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
+
+# +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 
 class Calculation(object):
     """ Base calculation class. All other calculation classes should be inherited
         from this class.
     """
 
-    #-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#
+    # -#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#
 
-    def __init__(self, prog, prmtop, incrd, inptraj, input_file, output):
+    def __init__(self, prog, prmtop, incrd, inptraj, input_file, output, xvv=None):
         self.prmtop = str(prmtop)
         self.incrd = incrd
         self.input_file = input_file
         self.inptraj = inptraj
         self.output = output
         self.program = prog
+        self.xvv = xvv
 
-        self.calc_setup = False # This means that the setup has run successfully
+        self.calc_setup = False  # This means that the setup has run successfully
 
         self.command_args = [self.program]
 
@@ -161,7 +164,7 @@ class Calculation(object):
             if own_handleo: process_stdout.close()
             if own_handlee: process_stdout.close()
 
-    #-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#
+    # -#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#
 
     def setup(self):
         """ Sets up the Calculation. Finds the program and adds that to the
@@ -169,15 +172,17 @@ class Calculation(object):
             method first, but then do anything else that is necessary for that
             calculation.
         """
-        self.calc_setup= True
+        self.calc_setup = True
 
-#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
+
+# +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 
 class EnergyCalculation(Calculation):
     """ Uses mmpbsa_py_energy to evaluate energies """
-    def __init__(self, prog, prmtop, incrd, inptraj, input_file, output, restrt):
+
+    def __init__(self, prog, prmtop, incrd, inptraj, input_file, output, restrt, xvv=None):
         Calculation.__init__(self, prog, prmtop, incrd, inptraj,
-                             input_file, output)
+                             input_file, output, xvv)
         self.restrt = restrt
 
     def setup(self):
@@ -186,14 +191,16 @@ class EnergyCalculation(Calculation):
         for the MPI version (since one is *always* written and you don't want 2
         threads fighting to write the same dumb file)
         """
-        self.command_args.append('-O')                    # overwrite flag
-        self.command_args.extend(('-i', self.input_file)) # input file flag
-        self.command_args.extend(('-p', self.prmtop))     # prmtop flag
-        self.command_args.extend(('-c', self.incrd))      # input coordinate flag
-        self.command_args.extend(('-y', self.inptraj))    # input trajectory flag
-        self.command_args.extend(('-o', self.output))     # output file flag
+        self.command_args.append('-O')  # overwrite flag
+        self.command_args.extend(('-i', self.input_file))  # input file flag
+        self.command_args.extend(('-p', self.prmtop))  # prmtop flag
+        self.command_args.extend(('-c', self.incrd))  # input coordinate flag
+        self.command_args.extend(('-y', self.inptraj))  # input trajectory flag
+        self.command_args.extend(('-o', self.output))  # output file flag
         if self.restrt is not None:
             self.command_args.extend(('-r', self.restrt))  # restart file flag
+        if self.xvv is not None:
+            self.command_args.extend(('-xvv', self.xvv))  # xvv file flag
 
         # Now test to make sure that the input file exists, since that's the only
         # one that may be absent (due to the use of -use-mdins)
@@ -202,12 +209,13 @@ class EnergyCalculation(Calculation):
 
         self.calc_setup = True
 
-#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
+
+# +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 
 class RISMCalculation(Calculation):
     """ This class handles RISM calculations """
 
-    #-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#
+    # -#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#
 
     def __init__(self, prog, prmtop, incrd, inptraj, xvvfile, output, INPUT):
         """ Sets up a RISM calculation. It's not as similar to the base class as
@@ -217,77 +225,114 @@ class RISMCalculation(Calculation):
         Calculation.__init__(self, prog, prmtop, incrd, inptraj, None, output)
 
         # Set up instance variables
-        self.xvvfile      = xvvfile
-        self.closure      = INPUT['closure']
-        self.polardecomp  = INPUT['polardecomp']
-        self.ng           = INPUT['ng'].replace(' ','') # get rid of spaces
-        self.solvbox      = INPUT['solvbox']
-        self.buffer       = INPUT['buffer']
-        self.grdspc       = str(INPUT['grdspc']).replace(' ','')
-        self.solvcut      = INPUT['solvcut']
-        self.tolerance    = INPUT['tolerance']
-        self.verbose      = INPUT['rism_verbose']
-        self.solvbox      = INPUT['solvbox']
-        self.gf           = INPUT['rismrun_gf']
+        self.xvvfile = xvvfile
+        self.closure = ','.join(map(str, INPUT['closure']))
+        self.polardecomp = INPUT['polardecomp']
+        self.ng = ','.join(map(str, INPUT['ng']))
+        self.solvbox = ','.join(map(str, INPUT['solvbox']))
+        self.buffer = INPUT['buffer']
+        self.grdspc = ','.join(map(str, INPUT['grdspc']))
+        self.solvcut = INPUT['solvcut']
+        self.tolerance = ','.join(map(str, INPUT['tolerance']))
+        self.verbose = INPUT['rism_verbose']
+        self.solvbox = ','.join(map(str, INPUT['solvbox']))
+        self.gf = INPUT['rismrun_gf']
 
-    #-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#
+        self.noasympcorr = INPUT['noasympcorr']
+        self.mdiis_del = INPUT['mdiis_del']
+        self.mdiis_restart = INPUT['mdiis_restart']
+        self.mdiis_nvec = INPUT['mdiis_nvec']
+        self.maxstep = INPUT['maxstep']
+        self.npropagate = INPUT['npropagate']
+        # self.centering = INPUT['centering']
+        # self.entropicDecomp = INPUT['entropicDecomp']
+        # self.pc_plus = INPUT['rismrun_pc+']
+        # self.uccoeff = ','.join(map(str, INPUT['uccoeff']))
+        self.treeDCF = INPUT['treeDCF']
+        self.treeTCF = INPUT['treeTCF']
+        self.treeCoulomb = INPUT['treeCoulomb']
+        self.treeDCFOrder = INPUT['treeDCFOrder']
+        self.treeTCFOrder = INPUT['treeTCFOrder']
+        self.treeCoulombOrder = INPUT['treeCoulombOrder']
+        self.treeDCFN0 = INPUT['treeDCFN0']
+        self.treeTCFN0 = INPUT['treeTCFN0']
+        self.treeCoulombN0 = INPUT['treeCoulombN0']
+        self.treeDCFMAC = INPUT['treeDCFMAC']
+        self.treeTCFMAC = INPUT['treeTCFMAC']
+        self.treeCoulombMAC = INPUT['treeCoulombMAC']
+        self.asympKSpaceTolerance = INPUT['asympKSpaceTolerance']
+        self.ljTolerance = INPUT['ljTolerance']
+
+    # -#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#
 
     def setup(self):
         """ Sets up the RISM calculation. All it has to do is fill in the
             necessary command-line arguments
         """
         # Set up some defaults
-
-        if self.ng == "-1,-1,-1":
-            ngflag = False
-        else:
-            ngflag = True
-
-        if self.solvbox == "-1,-1,-1":
-            solvboxflag = False
-        else:
-            solvboxflag = True
-
-        if self.polardecomp:
-            polardecompflag = True
-        else:
-            polardecompflag = False
+        ngflag = self.ng != "-1,-1,-1"
+        solvboxflag = self.solvbox != "-1,-1,-1"
+        polardecompflag = bool(self.polardecomp)
+        gfflag = bool(self.gf)
+        # pc_plusflag = bool(self.pc_plus)
 
         Calculation.setup(self)
-        self.command_args.extend( ('--xvv', self.xvvfile,
-                                   '--closure', self.closure,
-                                   '--buffer', self.buffer,
-                                   '--grdspc', self.grdspc,
-                                   '--solvcut', self.solvcut,
-                                   '--tolerance', self.tolerance,
-                                   '--verbose', self.verbose,
-                                   '--prmtop', self.prmtop,
-                                   '--pdb', self.incrd,
-                                   '--traj', self.inptraj))
+        self.command_args.extend(('--xvv', self.xvvfile,
+                                  '--closure', self.closure,
+                                  '--buffer', self.buffer,
+                                  '--grdspc', self.grdspc,
+                                  '--solvcut', self.solvcut,
+                                  '--tolerance', self.tolerance,
+                                  '--verbose', self.verbose,
+                                  '--prmtop', self.prmtop,
+                                  '--pdb', self.incrd,
+                                  '--traj', self.inptraj))
         if ngflag:
             self.command_args.extend(('--ng', self.ng))
         if solvboxflag:
             self.command_args.extend(('--solvbox', self.solvbox))
         if polardecompflag:
             self.command_args.extend(['--polarDecomp'])
-        if self.gf:
+        if gfflag:
             self.command_args.extend(['--gf'])
+        # if pc_plusflag:
+        #     self.command_args.extend(['--pc+'])
         if not os.path.exists(self.xvvfile):
             raise IOError('XVVFILE (%s) does not exist!' % self.xvvfile)
 
+        # additional variables
+        var_names = [self.mdiis_del, self.mdiis_restart, self.mdiis_nvec, self.maxstep, self.npropagate,
+                     self.treeDCF, self.treeTCF, self.treeCoulomb,
+                     self.treeDCFOrder, self.treeTCFOrder, self.treeCoulombOrder, self.treeDCFN0,
+                     self.treeTCFN0, self.treeCoulombN0, self.treeDCFMAC, self.treeTCFMAC, self.treeCoulombMAC,
+                     self.asympKSpaceTolerance, self.ljTolerance]
+
+        var_input_names = ['mdiis_del', 'mdiis_restart', 'mdiis_nvec', 'maxstep', 'npropagate',
+                           'treeDCF', 'treeTCF', 'treeCoulomb',
+                           'treeDCFOrder', 'treeTCFOrder', 'treeCoulombOrder', 'treeDCFN0', 'treeTCFN0',
+                           'treeCoulombN0', 'treeDCFMAC', 'treeTCFMAC', 'treeCoulombMAC',
+                           'asympKSpaceTolerance', 'ljTolerance']
+
+        for i in zip(var_names, var_input_names):
+            if i[1] not in ['treeDCF', 'treeTCF', 'treeCoulomb']:
+                self.command_args.extend((f'--{i[1]}', str(i[0])))
+            elif i[0] != 0:
+                self.command_args.extend([f'--{i[1]}'])
+
         self.calc_setup = True
 
-    #-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#
+    # -#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#
 
     def run(self, rank, *args, **kwargs):
         Calculation.run(self, rank, stdout=self.output % rank)
 
-#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
+
+# +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 
 class NmodeCalc(Calculation):
     """ Calculates entropy contribution by normal mode approximation """
 
-    #-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#
+    # -#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#
 
     def __init__(self, prog, prmtop, incrd, inptraj, output, INPUT):
         """ Initializes the nmode calculation. Need to set the options string """
@@ -297,18 +342,16 @@ class NmodeCalc(Calculation):
         kappa = sqrt(0.10806 * INPUT['nmode_istrng'])
         if INPUT['nmode_igb']:
             option_string = ('ntpr=10000, diel=C, kappa=%f, cut=1000, gb=1, ' +
-                             'dielc=%f, temp0=%f') % (kappa,
-                                                      INPUT['dielc'], INPUT['temp'])
+                             'dielc=%f, temp0=%f') % (kappa, INPUT['dielc'], INPUT['temperature'])
         else:
             option_string = ('ntpr=10000, diel=R, kappa=%f, cut=1000, gb=0, ' +
-                             'dielc=%f, temp0=%f') % (kappa,
-                                                      INPUT['dielc'], INPUT['temp'])
+                             'dielc=%f, temp0=%f') % (kappa, INPUT['dielc'], INPUT['temperature'])
 
         self.option_string = option_string
         self.drms = INPUT['drms']
         self.maxcyc = INPUT['maxcyc']
 
-    #-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#
+    # -#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#
 
     def setup(self):
         """ Sets up the simulation """
@@ -317,12 +360,13 @@ class NmodeCalc(Calculation):
                                   self.option_string, self.inptraj))
         self.calc_setup = True
 
-    #-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#
+    # -#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#
 
     def run(self, rank, *args, **kwargs):
         Calculation.run(self, rank, stdout=self.output % rank)
 
-#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
+
+# +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 
 class QuasiHarmCalc(Calculation):
     """ Quasi-harmonic entropy calculation class """
@@ -335,9 +379,9 @@ class QuasiHarmCalc(Calculation):
         self.stability = not bool(receptor_mask) and not bool(ligand_mask)
         self.receptor_mask, self.ligand_mask = receptor_mask, ligand_mask
         self.calc_setup = False
-        self.fnpre = fnpre # file name prefix
+        self.fnpre = fnpre  # file name prefix
 
-    #-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#
+    # -#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#
 
     def setup(self):
         """ Sets up a Quasi-harmonic calculation """
@@ -353,9 +397,9 @@ class QuasiHarmCalc(Calculation):
 
         # First thing we need is the average PDB as a reference
         ptraj_str = 'trajin %s\naverage %savgcomplex.pdb pdb chainid " "\ngo' % (self.inptraj,
-                                                                     prefix)
+                                                                                 prefix)
 
-        outfile = open(self.fnpre + 'create_average.out','w')
+        outfile = open(self.fnpre + 'create_average.out', 'w')
 
         process = Popen([self.program, self.prmtop], stdin=PIPE, stdout=outfile)
         out, err = process.communicate(ptraj_str.encode())
@@ -371,18 +415,20 @@ class QuasiHarmCalc(Calculation):
 
         self.calc_setup = True
 
-    #-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#
+    # -#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#
 
     def run(self, rank, *args, **kwargs):
         Calculation.run(self, rank, stdout=self.output)
 
-#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
+
+# +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 
 class PBEnergyCalculation(EnergyCalculation):
     """
     Specially handle the PB calculations to extract warnings and errors PBSA
     prints to stdout and redirect them to the user
     """
+
     def run(self, rank, stdout=sys.stdout, stderr=sys.stderr):
         """
         Runs the program. All command-line arguments must be set before calling
@@ -433,12 +479,13 @@ class PBEnergyCalculation(EnergyCalculation):
                 GMXMMPBSA_ERROR('%s failed with prmtop %s!\n\t' % (self.program, self.prmtop) +
                                 '\n\t'.join(error_list) + '\n' +
                                 'If you are using sander and PB calculation, check the *.mdout files to get the sander '
-                                'error and check this thread (http://archive.ambermd.org/201303/0548.html)\n',
+                                'error\n',
                                 CalcError)
         finally:
             if own_handle: process_stderr.close()
 
-#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
+
+# +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 
 class SurfCalc(Calculation):
     """
@@ -481,7 +528,8 @@ class SurfCalc(Calculation):
             raise CalcError('%s failed with prmtop %s!' % (self.program,
                                                            self.prmtop))
 
-#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
+
+# +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 
 class LcpoCalc(SurfCalc):
     """
@@ -495,7 +543,8 @@ class LcpoCalc(SurfCalc):
         output = self.output % rank
         return "trajin %s\nsolvent none\nsurf :* out %s\n" % (inptraj, output)
 
-#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
+
+# +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 
 class MolsurfCalc(SurfCalc):
     """ Uses molsurf to calculate the surface area """
@@ -511,13 +560,15 @@ class MolsurfCalc(SurfCalc):
         return "trajin %s\nmolsurf :* out %s probe %s offset %s" % (inptraj,
                                                                     output, self.probe, self.offset)
 
-#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
+
+# +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 
 class CopyCalc(Calculation):
     """
     This is for mutant files that are unchanged from 'normal' files (i.e., when
     the mutation is in the receptor, the ligand outputs are copied)
     """
+
     def __init__(self, orig_name, final_name):
         self.orig_name = orig_name
         self.final_name = final_name
@@ -537,69 +588,136 @@ class CopyCalc(Calculation):
 
         copy(orig_name, final_name)
 
-#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
+
+# +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 
 class PrintCalc(Calculation):
     """
     This is just a way to insert a printed message to the screen during the
     calculation list execution
     """
+
     def __init__(self, message):
         self.message = message
 
     def run(self, rank, stdout=sys.stdout, stderr=sys.stderr):
-        if rank == 0: stdout.write(self.message + '\n')
+        if rank == 0:
+            logging.info(self.message)
+            # stdout.write(self.message + '\n')
 
-#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
+
+# +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 
 class InteractionEntropyCalc:
     """
     Class for Interaction Entropy calculation
     :return {IE_key: data}
     """
-    def __init__(self, ggas, app, output):
+
+    def __init__(self, ggas, INPUT, iesegment=None):
+        """
+
+        Args:
+            ggas: Model GGAS energy
+            INPUT: INPUT dict
+            iesegment: If not defined, iesegment = INPUT['ie_segment']
+        """
         self.ggas = ggas
-        self.app = app
-        self.output = output
-        self.data = {}
+        self.INPUT = INPUT
+        self.isegment = iesegment or INPUT['ie_segment']
+        self.data = []
 
         self._calculate()
-        self.save_output()
 
     def _calculate(self):
-        # boltzmann constant
+        # boltzmann constant in kcal/(mol⋅K)
         k = 0.001985875
-        temp = self.app.INPUT['temperature']
+        temperature = self.INPUT['temperature']
 
-        energy_int = np.array([], dtype=np.float)
-        a_energy_int = np.array([], dtype=np.float)
         exp_energy_int = np.array([], dtype=np.float)
-        ts = np.array([], dtype=np.float)
-        for eint in self.ggas:
-            energy_int = np.append(energy_int, eint)
-            aeint = energy_int.mean()
-            a_energy_int = np.append(a_energy_int, aeint)
-            deint = eint - aeint
-            eceint = math.exp(deint / (k * temp))
+        self.data = np.zeros(self.ggas.size, dtype=np.float)
+
+        for i in range(self.ggas.size):
+            aeint = self.ggas[:i+1].mean()
+            deint = self.ggas[i] - aeint
+            try:
+                eceint = math.exp(deint / (k * temperature))
+            except CalcError:
+                logging.warning('The internal energy of your system has very large energy fluctuation so it is not '
+                                'possible to continue with the calculations. Please, make sure your system is '
+                                'consistent')
+                logging.info('The Interaction Entropy will be skipped...')
+                self.INPUT['interaction_entropy'] = 0
+                break
             exp_energy_int = np.append(exp_energy_int, eceint)
             aeceint = exp_energy_int.mean()
-            cts = k * temp * math.log(aeceint)
-            ts = np.append(ts, cts)
-        self.data = ts
-        self.ieframes = math.ceil(self.app.numframes * (self.app.INPUT['ie_segment'] / 100))
-        self.iedata = self.data[-self.ieframes:]
-        self.frames = [x for x in range(self.app.INPUT['startframe'],
-                                        self.app.INPUT['startframe'] + self.app.numframes * self.app.INPUT['interval'],
-                                        self.app.INPUT['interval'])]
+            cts = k * temperature * math.log(aeceint)
+            self.data[i] = cts
 
-    def save_output(self):
-        with open(self.output, 'w') as out:
-            out.write(f'Calculation for last {self.ieframes} frames:\n')
-            out.write(f'Interaction Entropy (-TΔS): {self.iedata.mean():.4f} +/- {self.iedata.std():.4f}\n\n')
-            out.write(f'Interaction Entropy per-frame:\n')
+        numframes = len(self.data)
+        self.ie_std = float(self.ggas.std())
+        self.ieframes = math.ceil(numframes * (self.isegment / 100))
+        self.iedata = self.data[-self.ieframes:]
+
+    def save_output(self, filename):
+        frames = list(
+            range(
+                self.INPUT['startframe'],
+                self.INPUT['startframe'] + len(self.data) * self.INPUT['interval'],
+                self.INPUT['interval']
+            )
+        )
+        with open(filename, 'w') as out:
+            out.write('| Interaction Entropy results\n')
+            out.write(f'IE-frames: last {self.ieframes}\n')
+            out.write(f'Internal Energy SD (sigma): {self.ie_std:9.2f}\n')
+            out.write(f'| Interaction Entropy (-TΔS): {self.iedata.mean():9.2f} +/- {self.iedata.std():7.2f}\n\n')
+            out.write('| Interaction Entropy per-frame:\n')
 
             out.write('Frame # | IE value\n')
-            for f, d in zip(self.frames, self.data):
-                out.write('{:d}  {:.2f}\n'.format(f,d))
+            for f, d in zip(frames, self.data):
+                out.write('{:d}  {:.2f}\n'.format(f, d))
 
-#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
+# +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
+
+class C2EntropyCalc:
+    """
+    Class for Interaction Entropy calculation
+    :return {IE_key: data}
+    """
+
+    def __init__(self, ggas, INPUT):
+        self.ggas = ggas
+        self.INPUT = INPUT
+
+        self._calculate()
+
+    def _calculate(self):
+        # gas constant in kcal/(mol⋅K)
+        R = 0.001987
+        temperature = self.INPUT['temperature']
+        self.ie_std = float(self.ggas.std())
+        self.c2data = (self.ie_std ** 2) / (2 * temperature * R)
+
+        size = self.ggas.size
+        array_of_c2 = np.zeros(2000)
+        for i in range(2000):
+            idxs = np.random.randint(0, size, size)
+            ie_std = self.ggas[idxs].std()
+            c2data = (ie_std ** 2) / (2 * temperature * R)
+            array_of_c2[i] = c2data
+
+        self.c2_std = float(np.sort(array_of_c2)[100:1900].std())
+        self.c2_ci = np.percentile(np.sort(array_of_c2)[100:1900], [2.5, 97.5])
+
+    def save_output(self, filename):
+        with open(filename, 'w') as out:
+            out.write('| C2 Entropy results\n')
+            out.write(f'C2 Entropy (-TΔS): {self.c2data:.4f}\n')
+            out.write(f'C2 Entropy SD: {self.c2_std:.4f}\n')
+            out.write(f'Internal Energy SD (sigma): {self.ie_std:9.2f}\n')
+            out.write(f'C2 Entropy CI: {self.c2_ci[0]:.4f} {self.c2_ci[1]:.4f}\n')
+
+
+
+# +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
