@@ -25,12 +25,14 @@ in gmx_MMPBSA will be assigned as attributes to the returned class.
 # ##############################################################################
 
 import os
+import sys
 from argparse import ArgumentParser, ArgumentDefaultsHelpFormatter, RawDescriptionHelpFormatter, RawTextHelpFormatter
 from pathlib import Path
 from GMXMMPBSA import __version__, __mmpbsa_version__, __ambertools_version__
 from GMXMMPBSA.exceptions import GMXMMPBSA_ERROR
+from types import SimpleNamespace
 
-class OptionList(object):
+class OptionList(SimpleNamespace):
     """
     Just a container to hold the command-line options. Necessary when reading in
     a gmx_MMPBSA info file to have a container to load the results from the
@@ -40,9 +42,14 @@ class OptionList(object):
 
 
 if os.getenv('AMBERHOME'):
-    rism = os.path.join(os.getenv('AMBERHOME'), 'dat', 'mmpbsa', 'spc.xvv')
+    # rism_xvv = os.path.join(os.getenv('AMBERHOME'), 'dat', 'mmpbsa', 'spc.xvv')
+    path_to_file = os.path.join(os.getenv('AMBERHOME'), 'AmberTools', 'test', 'rism1d', 'tip3p-kh', 'tip3p.xvv.save')
+    if os.path.exists(path_to_file):
+        rism_xvv = path_to_file
+    else:
+        rism_xvv = os.path.join(Path(__file__).parent.joinpath('data'), 'xvv_files', 'tip3p.xvv')
 else:
-    rism = None
+    rism_xvv = os.path.join(Path(__file__).parent.joinpath('data'), 'xvv_files', 'tip3p.xvv')
 
 
 def check_arg(str_suffix, path=False):
@@ -51,16 +58,14 @@ def check_arg(str_suffix, path=False):
             result = f(*args)
             if path and not Path(result).exists():
                 GMXMMPBSA_ERROR(f'{result} do not exist or is inaccessible')
-            if not Path(result).suffix in str_suffix:
+            if Path(result).suffix not in str_suffix:
                 GMXMMPBSA_ERROR(f'{result} does not correspond to the required structure format {str_suffix}')
             return result
-
         return gmx_MMPBSA_file
-
     return decorator
 
 
-@check_arg(['.tpr', '.pdb', '.gro'], True)
+@check_arg(['.tpr', '.pdb'], True)
 def structure(arg):
     return arg
 
@@ -90,11 +95,18 @@ def index(arg):
     return arg
 
 
+class GMXMMPBSA_ArgParser(ArgumentParser):
+
+    def exit(self, status=0, message=None):
+        if message:
+            GMXMMPBSA_ERROR(message)
+        sys.exit(status)
+
+
 description = ("gmx_MMPBSA is a new tool based on AMBER's MMPBSA.py aiming to perform end-state free energy calculations"
                " with GROMACS files. This program is an adaptation of Amber's MMPBSA.py and essentially works as such. "
-               "As gmx_MMPBSA adapts MMPBSA.py, since it has all the resources of this script and work with any "
-               "GROMACS version. This program will calculate binding free energies using end-state free energy "
-               "methods on an ensemble of snapshots using a variety of implicit solvent models.")
+               "gmx_MMPBSA works with any GROMACS version. This program will calculate binding free energies using "
+               "end-state free energy methods on an ensemble of snapshots using a variety of implicit solvent models.")
 
 complex_group_des = ("Complex files and info that are needed to perform the calculation. If the receptor and/or the "
                      "ligand info is not defined, we generate them from that of the complex.")
@@ -107,7 +119,7 @@ ligand_group_des = '''Ligand files and info that are needed to perform the calcu
 # Set up the MM/PBSA parser here. It clutters up the MMPBSA_App to do it there
 
 # noinspection PyTypeChecker
-parser = ArgumentParser(epilog=f'''gmx_MMPBSA is an effort to implement the GB/PB and others calculations in GROMACS.
+parser = GMXMMPBSA_ArgParser(epilog=f'''gmx_MMPBSA is an effort to implement the GB/PB and others calculations in GROMACS.
                                     \nBased on MMPBSA.py (version {__mmpbsa_version__}) and 
                                     AmberTools{__ambertools_version__}''',
                         description=(description + '''This is the core of gmx_MMPBSA and it will do all the 
@@ -119,6 +131,8 @@ parser.add_argument('-v', '--version', action='version',
 parser.add_argument('--input-file-help', dest='infilehelp', action='store_true',
                     help='Print all available options in the input file.',
                     default=False)
+parser.add_argument('--create_input', dest='createinput', choices=['gb', 'pb', 'rism', 'ala', 'decomp', 'nmode', 'all'],
+                    nargs='*', help='Create an new input file with selected calculation type.')
 group = parser.add_argument_group('Miscellaneous Options')
 group.add_argument('-O', '--overwrite', default=False, action='store_true',
                    help='Allow output files to be overwritten', dest='overwrite')
@@ -128,7 +142,7 @@ group.add_argument('-prefix', dest='prefix', default='_GMXMMPBSA_',
 group = parser.add_argument_group('Input and Output Files', '''These options specify the input files and optional 
 output files.''')
 group.add_argument('-i', dest='input_file', metavar='FILE', help='MM/PBSA input file.')
-group.add_argument('-xvvfile', dest='xvvfile', help='XVV file for 3D-RISM.', default=rism)
+group.add_argument('-xvvfile', dest='xvvfile', help='XVV file for 3D-RISM.', default=rism_xvv)
 group.add_argument('-o', dest='output_file', default='FINAL_RESULTS_MMPBSA.dat', metavar='FILE',
                    help='Output file with MM/PBSA statistics.')
 group.add_argument('-do', dest='decompout', metavar='FILE', default='FINAL_DECOMP_MMPBSA.dat',
@@ -156,8 +170,7 @@ group.add_argument('-cs', dest='complex_tpr', metavar='<Structure File>', defaul
                    help='''Structure file of the complex. If it is Protein-Ligand 
                          (small molecule) complex and -cp is not defined, make 
                          sure that you define -lm option. See -lm description 
-                         below. Allowed formats: *.tpr (recommended), *.pdb, 
-                         *.gro''')
+                         below. Allowed formats: *.tpr (recommended), *.pdb''')
 group.add_argument('-ci', dest='complex_index', metavar='<Index File>', default=None, type=index,
                    help='Index file of the bound complex.')
 group.add_argument('-cg', dest='complex_groups', metavar='index', nargs=2, default=None, type=int,
@@ -179,7 +192,7 @@ group.add_argument('-cr', dest='reference_structure', metavar='<PDB File>', defa
 group = parser.add_argument_group('Receptor', receptor_group_des)
 group.add_argument('-rs', dest='receptor_tpr', metavar='<Structure File>', default=None, type=structure,
                    help='''Structure file of the unbound receptor for multiple trajectory approach.
-                         Allowed formats: *.tpr (recommended), *.pdb, *.gro''')
+                         Allowed formats: *.tpr (recommended), *.pdb''')
 group.add_argument('-ri', dest='receptor_index', metavar='<Index File>', default=None, type=index,
                    help='Index file of the unbound receptor.')
 group.add_argument('-rg', dest='receptor_group', metavar='index', default=None, type=int,
@@ -201,7 +214,7 @@ group.add_argument('-lm', dest='ligand_mol2', metavar='<Structure File>', defaul
                          force fields. Must be the Antechamber output *.mol2.''')
 group.add_argument('-ls', dest='ligand_tpr', metavar='<Structure File>', default=None, type=structure,
                    help='''Structure file of the unbound ligand. If ligand is a small molecule and -lp is not defined,
-                   make sure that you define above -lm option. Allowed formats: *.tpr (recommended), *.pdb, *.gro''')
+                   make sure that you define above -lm option. Allowed formats: *.tpr (recommended), *.pdb''')
 group.add_argument('-li', dest='ligand_index', metavar='<Index File>', type=index,
                    default=None, help='Index file of the unbound ligand. Only if tpr file was define in -ls.')
 group.add_argument('-lg', dest='ligand_group', metavar='index', default=None, type=int,
@@ -215,15 +228,7 @@ group.add_argument('-lp', dest='ligand_top', metavar='<Topology>', default=None,
                    help='''Topology file of the ligand.''')
 
 group = parser.add_argument_group('Miscellaneous Actions')
-group.add_argument('-make-mdins', dest='make_mdins', default=False,
-                   action='store_true', help='''Create the input files for each
-                  calculation and quit. This allows you to modify them and
-                  re-run using -use-mdins''')
-group.add_argument('-use-mdins', dest='use_mdins', default=False,
-                   action='store_true', help='''Use existing input files for each
-                  calculation. If they do not exist with the appropriate names,
-                  %(prog)s will quit in error.''')
-group.add_argument('-rewrite-output', dest='rewrite_output', default=False,
+group.add_argument('--rewrite-output', dest='rewrite_output', default=False,
                    action='store_true', help='''Do not re-run any calculations,
                   just parse the output files from the previous calculation and
                   rewrite the output files.''')
@@ -246,8 +251,7 @@ group.add_argument('-r', '--recursive', help='Search recursively in this folder
                    default=False)
 
 # tester parser
-description = ('This program is part of gmx_MMPBSA and will allow you to run the different gmx_MMPBSA calculations '
-              'examples easily.')
+description = ('This program is part of gmx_MMPBSA and will allow you to run various gmx_MMPBSA examples easily.')
 testparser = ArgumentParser(epilog=f'gmx_MMPBSA is an effort to implement the GB/PB and others calculations in '
                                   f'GROMACS. \nBased on MMPBSA.py (version {__mmpbsa_version__}) and '
                                   f'AmberTools{__ambertools_version__}',
@@ -256,39 +260,41 @@ testparser = ArgumentParser(epilog=f'gmx_MMPBSA is an effort to implement the GB
 testparser.add_argument('-v', '--version', action='version',
                        version='%%(prog)s %s based on MMPBSA version %s' % (__version__, __mmpbsa_version__))
 group = testparser.add_argument_group('Test options')
-group.add_argument('-t', dest='test', choices=['all', 'minimal', '3drism','ala_scan', 'decomp',
-                                            'prot_lig_mt', 'stability', 'ie', 'nmode', 'prot_prot', 'prot_lig_st',
-                                            'prot_dna', 'metalloprot_pep', 'prot_dna_rna_ions_lig', 'prot_glycan',
-                                            'memb_prot', 'prot_lig_charmm', 'memb_charmm'], default='minimal',
+group.add_argument('-t', dest='test', choices=list(range(19)) + [101], type=int, nargs='*', default=[2],
                    help='''\
-The level at which you want to take the test.
-* all - make all examples.
-* minimal - does a minimal test with a set of systems and analyzes that show 
-    that gmx_MMPBSA runs correctly. Only exclude 3drism and nmode because they
-    can take a long time and prot_lig_mt because it is quite similar to 
-    prot_lig_st
+The level the test is going to be run at. Multiple systems and analysis can be run at the same time.
+      Nr. of Sys  
+* 0      16     All -- Run all examples (Can take a long time!!!)
+* 1      13     Minimal -- Does a minimal test with a set of systems and analyzes 
+                that show that gmx_MMPBSA runs correctly. Only exclude 3drism, nmode
+                protein-ligand MT because take a long time or are redundant
+* 2       9     Fast -- Only the calculations that take a short time are run (Default)
 [Systems]:
-* prot_lig_st             Protein-Ligand (Single trajectory approximation)
-* prot_prot               Protein-Protein
-* prot_dna                Protein-DNA
-* memb_prot               Protein-Membrane
-* prot_glycan             Protein-Glycan
-* metalloprot_pep         Metalloprotein-Peptide
-* prot_dna_rna_ions_lig   Protein-DNA-RNA-IONs-Ligand
-* prot_lig_charmm         Protein-Ligand (CHARMM force field)
-* memb_charmm             Protein-ligand complex in membrane with CHARMMff 
+     Slow Frames
+* 3    . | 10   Protein-Ligand (Single trajectory approximation)
+* 4    . | 10   Protein-Protein
+* 5    . | 10   Protein-DNA
+* 6    x |  4   Protein-Membrane
+* 7    . | 10   Protein-Glycan
+* 8    x |  4   Metalloprotein-Peptide
+* 9    . | 10   Protein-DNA-RNA-IONs-Ligand
+* 10   x |  4   Protein-Ligand (CHARMM force field)
+* 11   x |  4   Protein-ligand complex in membrane with CHARMMff 
 [Analysis]:
-* ala_scan                Alanine Scanning
-* stability               Stability calculation
-* decomp                  Decomposition Analysis
-* prot_lig_mt             Protein-Ligand (Multiple trajectory approximation)
-* ie                      Interaction Entropy approximation
-* nmode                   Entropy calculation using Normal Mode approximation 
-* 3drism                  Calculations using 3D-RISM approximation
+     Slow Frames
+* 12   . | 10   Alanine Scanning
+* 13   . | 10   Stability calculation
+* 14   . | 10   Decomposition Analysis
+* 15   . | 16   Interaction Entropy approximation
+* 16   . | 10   Protein-Ligand (Multiple trajectory approximation)
+* 17   x |  4   Entropy calculation using Normal Mode approximation 
+* 18   x |  4   Calculations using 3D-RISM approximation
 ''')
 group.add_argument('-f', '--folder', help='Defines the folder to store all data', type=Path, default='.')
+group.add_argument('-r', '--reuse', help='Defines the existing test forlder will be reuse', action='store_true')
 group.add_argument('-ng', '--nogui', help='No open gmx_MMPBSA_ana after all calculations finished',
                    action='store_true',  default=False)
-group.add_argument('-n', '--num_processors', type=int, default=1,
-                   help='Defines the number of processor cores you want to use since gmx_MMPBSA in this test uses a '
-                        'single processor')
+group.add_argument('-n', '--num_processors', type=int, default=4,
+                   help='Defines the number of processor cores you want to use with MPI per calculation. If the number '
+                        'of frames is less than the number of cpus defined, the calculation will be performed with '
+                        'the number of processors = number of frames')
