@@ -37,8 +37,8 @@ from warnings import warn
 from GMXMMPBSA.exceptions import (TrajError, MMPBSA_Error, InternalError,
                                   MutantResError)
 
+strip_mask = ':WAT,Cl*,CIO,Cs+,IB,K*,Li+,MG*,Na+,Rb+,CS,RB,NA,F,CL'
 
-# +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 
 def make_trajectories(INPUT, FILES, size, cpptraj, pre):
     """
@@ -60,7 +60,7 @@ def make_trajectories(INPUT, FILES, size, cpptraj, pre):
     #    traj.Setup(INPUT['startframe'], INPUT['endframe'], INPUT['interval'])
     #    if not stability:
     #       traj.Image()
-    #    traj.StripSolvent(INPUT['strip_mask'])
+    #    traj.StripSolvent(strip_mask)
     # # If we do not have a solvated topology...
     # else:
     #    print FILES.complex_prmtop, FILES.complex_trajs, cpptraj
@@ -68,9 +68,11 @@ def make_trajectories(INPUT, FILES, size, cpptraj, pre):
     traj = Trajectory(FILES.complex_prmtop, FILES.complex_trajs, cpptraj)
     traj.Setup(INPUT['startframe'], INPUT['endframe'], INPUT['interval'])
     # RMS fit
-    traj.rms('!(%s)' % INPUT['strip_mask'])
+    traj.rms('!(%s)' % strip_mask)
 
-    num_frames_total = traj.processed_frames
+    com_frames = int(traj.processed_frames)
+    rec_frames = 0
+    lig_frames = 0
     num_frames_nmode = 0
 
     # Sanity check
@@ -115,7 +117,7 @@ def make_trajectories(INPUT, FILES, size, cpptraj, pre):
                          frames=frame_string, filetype=INPUT['netcdf'])
             last_frame += frame_count[i]
         traj.Unstrip(restrip_solvent=True)
-        traj.rms('!(%s)' % INPUT['strip_mask'])
+        traj.rms('!(%s)' % strip_mask)
 
     if not stability and not FILES.ligand_trajs:
         traj.Strip(INPUT['receptor_mask'])
@@ -130,7 +132,7 @@ def make_trajectories(INPUT, FILES, size, cpptraj, pre):
                          frames=frame_string, filetype=INPUT['netcdf'])
             last_frame += frame_count[i]
         traj.Unstrip(restrip_solvent=True)
-        traj.rms('!(%s)' % INPUT['strip_mask'])
+        traj.rms('!(%s)' % strip_mask)
 
     # Run cpptraj to get the trajectory
     traj.Run(pre + 'normal_traj_cpptraj.out')
@@ -143,12 +145,12 @@ def make_trajectories(INPUT, FILES, size, cpptraj, pre):
         #    rectraj = Trajectory(FILES.solvated_receptor_prmtop,
         #                         FILES.receptor_trajs, cpptraj)
         #    rectraj.Setup(INPUT['startframe'],INPUT['endframe'],INPUT['interval'])
-        #    rectraj.StripSolvent(INPUT['strip_mask'])
+        #    rectraj.StripSolvent(strip_mask)
         # else:
-        rectraj = Trajectory(FILES.receptor_prmtop,
-                             FILES.receptor_trajs, cpptraj)
+        rectraj = Trajectory(FILES.receptor_prmtop, FILES.receptor_trajs, cpptraj)
         rectraj.Setup(INPUT['startframe'], INPUT['endframe'], INPUT['interval'])
-        rectraj.rms('!(%s)' % INPUT['strip_mask'])
+        rec_frames = int(rectraj.processed_frames)
+        rectraj.rms('!(%s)' % strip_mask)
         if INPUT['full_traj'] or INPUT['qh_entropy']:
             rectraj.Outtraj(pre + 'receptor.%s' % trj_suffix,
                             filetype=INPUT['netcdf'])
@@ -182,11 +184,12 @@ def make_trajectories(INPUT, FILES, size, cpptraj, pre):
         #    ligtraj = Trajectory(FILES.solvated_ligand_prmtop,
         #                         FILES.ligand_trajs, cpptraj)
         #    ligtraj.Setup(INPUT['startframe'],INPUT['endframe'],INPUT['interval'])
-        #    ligtraj.StripSolvent(INPUT['strip_mask'])
+        #    ligtraj.StripSolvent(strip_mask)
         # else:
         ligtraj = Trajectory(FILES.ligand_prmtop, FILES.ligand_trajs, cpptraj)
         ligtraj.Setup(INPUT['startframe'], INPUT['endframe'], INPUT['interval'])
-        ligtraj.rms('!(%s)' % INPUT['strip_mask'])
+        lig_frames = int(ligtraj.processed_frames)
+        ligtraj.rms('!(%s)' % strip_mask)
         ligtraj.Outtraj(pre + 'ligand.%s' % trj_suffix, filetype=INPUT['netcdf'])
         ligtraj.Outtraj(pre + 'ligand.pdb', frames='1', filetype='pdb')
         ligtraj.Outtraj(pre + 'dummyligand.inpcrd', frames='1',
@@ -216,10 +219,9 @@ def make_trajectories(INPUT, FILES, size, cpptraj, pre):
     if INPUT['nmoderun']:
         nmtraj = Trajectory(FILES.complex_prmtop, [pre + 'complex.%s.%d' %
                                                    (trj_suffix, i) for i in range(size)], cpptraj)
-        nmtraj.Setup(INPUT['nmstartframe'], INPUT['nmendframe'],
-                     INPUT['nminterval'])
+        nmtraj.Setup(INPUT['nmstartframe'], INPUT['nmendframe'], INPUT['nminterval'])
 
-        num_frames_nmode = nmtraj.processed_frames
+        num_frames_nmode = int(nmtraj.processed_frames)
 
         # Now split up the complex trajectory by thread
         if nmtraj.processed_frames < size:
@@ -242,9 +244,7 @@ def make_trajectories(INPUT, FILES, size, cpptraj, pre):
         if not stability:
             nmtraj = Trajectory(FILES.receptor_prmtop, [pre + 'receptor.%s.%d' %
                                                         (trj_suffix, i) for i in range(size)], cpptraj)
-            nmtraj.Setup(INPUT['nmstartframe'], INPUT['nmendframe'],
-                         INPUT['nminterval'])
-
+            nmtraj.Setup(INPUT['nmstartframe'], INPUT['nmendframe'], INPUT['nminterval'])
             # Now split up the complex trajectory by thread
             if nmtraj.processed_frames < size:
                 raise MMPBSA_Error('More processors than receptor nmode frames!')
@@ -265,9 +265,7 @@ def make_trajectories(INPUT, FILES, size, cpptraj, pre):
 
             nmtraj = Trajectory(FILES.ligand_prmtop, [pre + 'ligand.%s.%d' %
                                                       (trj_suffix, i) for i in range(size)], cpptraj)
-            nmtraj.Setup(INPUT['nmstartframe'], INPUT['nmendframe'],
-                         INPUT['nminterval'])
-
+            nmtraj.Setup(INPUT['nmstartframe'], INPUT['nmendframe'], INPUT['nminterval'])
             # Now split up the complex trajectory by thread
             if nmtraj.processed_frames < size:
                 raise MMPBSA_Error('More processors than ligand nmode frames!')
@@ -289,8 +287,7 @@ def make_trajectories(INPUT, FILES, size, cpptraj, pre):
         # end if not stability
 
     # end if INPUT['nmoderun']
-
-    return int(num_frames_total), int(num_frames_nmode)
+    return com_frames, rec_frames, lig_frames, num_frames_nmode
 
 
 # +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
@@ -306,15 +303,21 @@ def make_mutant_trajectories(INPUT, FILES, rank, cpptraj,
 
     if INPUT['netcdf']:
         trj_suffix = 'nc'
-        raise TypeError('Alanine/Glycine scanning requires ASCII trajectories '
-                        '(netcdf=0)')
+        raise TypeError('Alanine/Glycine scanning requires ASCII trajectories (netcdf=0)')
     else:
         trj_suffix = 'mdcrd'
 
-    if not stability and not ((FILES.ligand_prmtop == FILES.mutant_ligand_prmtop
-                               and FILES.receptor_prmtop != FILES.mutant_receptor_prmtop) or (
-                                      FILES.ligand_prmtop != FILES.mutant_ligand_prmtop and
-                                      FILES.receptor_prmtop == FILES.mutant_receptor_prmtop)):
+    if (
+        not stability
+        and (
+            FILES.ligand_prmtop != FILES.mutant_ligand_prmtop
+            or FILES.receptor_prmtop == FILES.mutant_receptor_prmtop
+        )
+        and (
+            FILES.ligand_prmtop == FILES.mutant_ligand_prmtop
+            or FILES.receptor_prmtop != FILES.mutant_receptor_prmtop
+        )
+    ):
         raise MMPBSA_Error('Alanine/Glycine scanning requires either a mutated '
                            'ligand or receptor topology file with only 1 mutant residue, but not '
                            'both')
